@@ -14,33 +14,27 @@ See the License for the specific language governing permissions and
 limitations under the License.
 */
 
-#define _GNU_SOURCE /* dprintf() */
+#define _GNU_SOURCE /* dprintf(), asprintf */
 
 #include <libgen.h>
 #include <limits.h>
+#include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/resource.h>
+#include <sys/socket.h>
 #include <sys/time.h>
 #include <sys/types.h>
-#include <sys/socket.h>
-#include <sys/resource.h>
 #include <sys/un.h>
 #include <sysexits.h>
 #include <syslog.h>
 #include <time.h>
 #include <unistd.h>
-#include <stdarg.h>
 
 #include "subprocess.h"
 #include "tempdir.h"
 
-/* FreeBSD does not have HOST_NAME_MAX defined.
- * TODO(jaq): use sysconf() to discover its value. */
-#if !defined(HOST_NAME_MAX) && defined(_POSIX_HOST_NAME_MAX)
-#define HOST_NAME_MAX _POSIX_HOST_NAME_MAX
-#endif
-
 static void usage(char * prog) {
   fprintf(stderr,
           "Usage: %s [options] command [arg [arg] ...]\n\n"
@@ -56,8 +50,6 @@ static void usage(char * prog) {
           " -h       print this help\n", prog);
 }
 
-const char * TEMPLATE = ".XXXXXX";
-
 enum var_kind {
   GAUGE,
   ABSOLUTE
@@ -97,9 +89,9 @@ void add_variable(struct variable ** var_list, const char * name, const enum var
 int main(int argc, char ** argv) {
   char * progname;
   int arg;
-  char * collectd_sockname;
-  char * statistics_filename;
-  char * temp_filename;
+  char * collectd_sockname = NULL;
+  char * statistics_filename = NULL;
+  char * temp_filename = NULL;
   char * command;
   char ** command_args;
   struct timeval start_wall_time, end_wall_time;
@@ -111,35 +103,26 @@ int main(int argc, char ** argv) {
   int debug = 0;
   struct rusage ru;
   struct variable * var_list = NULL, * var;
-  size_t s_len;
 
   progname = argv[0];
 
   while ((arg = getopt(argc, argv, "+C:f:hd")) > 0) {
     switch (arg) {
     case 'C':
-      s_len = strlen(optarg) + 1;
-      collectd_sockname = malloc(s_len);
-      if (collectd_sockname == NULL) {
-        perror("Allocating space for collectd_sockname");
+      if (asprintf(&collectd_sockname, "%s", optarg) == -1) {
+        perror("asprintf collectd_sockname");
         exit(EX_OSERR);
       }
-      strncpy(collectd_sockname, optarg, s_len);
-      collectd_sockname[s_len] = '\0';
       break;
     case 'h':
       usage(progname);
       exit(EXIT_SUCCESS);
       break;
     case 'f':
-      s_len = strlen(optarg) + 1;
-      statistics_filename = malloc(s_len);
-      if (statistics_filename == NULL) {
-        perror("Allocating space for statistics_filename");
+      if (asprintf(&statistics_filename, "%s", optarg) == -1) {
+        perror("asprintf");
         exit(EX_OSERR);
       }
-      strncpy(statistics_filename, optarg, s_len);
-      statistics_filename[s_len] = '\0';
       break;
     case 'd':
       debug = LOG_PERROR;
@@ -173,27 +156,18 @@ int main(int argc, char ** argv) {
   gettimeofday(&end_wall_time, NULL);
 
 
-  if (statistics_filename[0] == '\0') {
-    strncat(statistics_filename, make_tempdir(),
-            PATH_MAX - strlen(statistics_filename) - 1);
-    strncat(statistics_filename, "/",
-            PATH_MAX - strlen(statistics_filename) - 1);
-    strncat(statistics_filename, command,
-            PATH_MAX - strlen(statistics_filename) - 1);
-    strncat(statistics_filename, ".stat",
-            PATH_MAX - strlen(statistics_filename) - 1);
-    statistics_filename[PATH_MAX-1] = '\0';
+  if (statistics_filename == NULL) {
+    if (asprintf(&statistics_filename, "%s/%s.stat", make_tempdir(), command) == -1) {
+      perror("asprintf");
+      exit(EX_OSERR);
+    }
   }
   syslog(LOG_DEBUG, "statistics filename is %s", statistics_filename);
 
-  s_len = strlen(statistics_filename) + strlen(TEMPLATE) + 1;
-  temp_filename = malloc(s_len);
-  if (temp_filename == NULL) {
-    perror("Allocation of temp_filename");
+  if (asprintf(&temp_filename, "%s.XXXXXX", statistics_filename) == -1) {
+    perror("asprintf");
     exit(EX_OSERR);
   }
-  strncpy(temp_filename, statistics_filename, s_len);
-  strncat(temp_filename, TEMPLATE, s_len - strlen(temp_filename) - 1);
   syslog(LOG_DEBUG, "temp filename is %s", temp_filename);
 
   if ((temp_fd = mkstemp(temp_filename)) < 0) {
@@ -279,7 +253,8 @@ int main(int argc, char ** argv) {
 
   /* Write to collectd */
   if (collectd_sockname != NULL) {
-    char hostname[HOST_NAME_MAX];
+    char * hostname;
+    long hostname_len;
     struct sockaddr_un sock;
     int s;
 
@@ -296,7 +271,16 @@ int main(int argc, char ** argv) {
       goto end;
     }
 
-    gethostname(hostname, HOST_NAME_MAX-1);
+    hostname_len = sysconf(_SC_HOST_NAME_MAX);
+    if (hostname_len <= 0) hostname_len = _POSIX_HOST_NAME_MAX;
+    if ((hostname = malloc(hostname_len)) == NULL) {
+      perror("malloc hostname");
+      exit(EX_OSERR);
+    }
+    if (gethostname(hostname, hostname_len) == -1) {
+      perror("gethostname");
+      exit(EX_OSERR);
+    }
 
     for (var = var_list; var != NULL; var = var->next) {
       char type[10] = {'\0'};
